import os
import sys
import struct

sys.path.append('/usr/local/actianzen/lib')
import btrievePython as btrv

def ljust(string,length):
    count_length = 0
    for char in string:
        if ord(char) <= 255:
            count_length += 1
        else:
            count_length += 2
    return string + (length-count_length) * ' '


btrieveFileName = "漢字です.mkd"
recordFormat = "<iB32sBBBH"
recordLength = 42
keyFormat = "<i"
key1Format = "B32s"

# Create a session:
btrieveClient = btrv.BtrieveClient(0x4232, 0) #B2

# Specify FileAttributes for the new file:
btrieveFileAttributes = btrv.BtrieveFileAttributes()
rc = btrieveFileAttributes.SetFixedRecordLength(recordLength)

# Specify Key 0 as an autoinc:
btrieveKeySegment = btrv.BtrieveKeySegment()
rc = btrieveKeySegment.SetField(0, 4, btrv.Btrieve.DATA_TYPE_AUTOINCREMENT)
btrieveIndexAttributes = btrv.BtrieveIndexAttributes()
rc = btrieveIndexAttributes.AddKeySegment(btrieveKeySegment)
rc = btrieveIndexAttributes.SetDuplicateMode(False)
rc = btrieveIndexAttributes.SetModifiable(True)

# Create the file:
rc = btrieveClient.FileCreate(btrieveFileAttributes, btrieveIndexAttributes,
btrieveFileName, btrv.Btrieve.CREATE_MODE_OVERWRITE)
if (rc == btrv.Btrieve.STATUS_CODE_NO_ERROR):
    print('File ', btrieveFileName, ' created successfully!')
else:
    print('File ', btrieveFileName, ' not created; ', rc, ': ', btrv.Btrieve.StatusCodeToString(rc))

# Allocate a file object:
btrieveFile = btrv.BtrieveFile()

# Open the file:
rc = btrieveClient.FileOpen(btrieveFile, btrieveFileName, None, btrv.Btrieve.OPEN_MODE_NORMAL)
if (rc == btrv.Btrieve.STATUS_CODE_NO_ERROR):
    print('File open successful!')
else:
    print('File open failed - status: ', rc, ': ', btrv.Btrieve.StatusCodeToString(rc))

# Create Key 1 as a String with Null Indicator Byte:
btrieveKey1aSegment = btrv.BtrieveKeySegment()
rc = btrieveKey1aSegment.SetField(4, 1, btrv.Btrieve.DATA_TYPE_NULL_INDICATOR_SEGMENT)
rc = btrieveKey1aSegment.SetDescendingSortOrder(True)
btrieveKey1bSegment = btrv.BtrieveKeySegment()
rc = btrieveKey1bSegment.SetField(5, 32, btrv.Btrieve.DATA_TYPE_CHAR)
btrieveIndex1Attributes = btrv.BtrieveIndexAttributes()
rc = btrieveIndex1Attributes.AddKeySegment(btrieveKey1aSegment)
rc = btrieveIndex1Attributes.AddKeySegment(btrieveKey1bSegment)
rc = btrieveIndex1Attributes.SetDuplicateMode(btrv.Btrieve.DUPLICATE_MODE_ALLOWED_NONREPEATING)
rc = btrieveIndex1Attributes.SetModifiable(True)
rc = btrieveFile.IndexCreate(btrieveIndex1Attributes)
if (rc == btrv.Btrieve.STATUS_CODE_NO_ERROR):
    print('Index 1 created successfully!')
else:
    print('Index 1 not created; error: ', rc, ': ', btrv.Btrieve.StatusCodeToString(rc))

# Insert records:
iinserting = True
print('\n')
while iinserting:
    new_name = input('Insert name (Q to quit): ' )
    if new_name.lower() == 'q':
        iinserting = False
    else:
        record = struct.pack(recordFormat, 0, 0, ljust(new_name,32).encode('cp932'), 0, 22, 1, 2018)
        rc = btrieveFile.RecordCreate(record)
        if (rc == btrv.Btrieve.STATUS_CODE_NO_ERROR):
            print(' Insert successful!')
        else:
            print(' Insert failed - status: ', rc, ': ', btrv.Btrieve.StatusCodeToString(rc))

# Get Record count:
btrieveFileInfo = btrv.BtrieveFileInformation()
rc = btrv.BtrieveFile.GetInformation(btrieveFile, btrieveFileInfo)
print('\nTotal Records inserted =', btrieveFileInfo.GetRecordCount())

# Display all records in sorted order
print('\nHere is a list of the names in alphabetical order:')
readLength = btrieveFile.RecordRetrieveFirst(1, record, 0)
while (readLength > 0):
    recordUnpacked = struct.unpack(recordFormat, record)
    print(' ID:', recordUnpacked[0], ' Name:', recordUnpacked[2].decode('cp932'))
    readLength = btrieveFile.RecordRetrieveNext(record, 0)

# Look up record by name
ireading = True
while ireading:
    find_name = input('\nFind name (Q to quit): ' )
    if find_name.lower() == 'q':
        ireading = False
    else:
        key1Value = struct.pack(key1Format, 0, ljust(find_name,32).encode('cp932'))
        readLength = btrieveFile.RecordRetrieve(btrv.Btrieve.COMPARISON_EQUAL, 1, key1Value, record)
        if (readLength > 0) :
            recordUnpacked = struct.unpack(recordFormat, record)
            print(' Matching record found: ID:', recordUnpacked[0], ' Name:', recordUnpacked[2].decode('cp932'))
        else:
            print(' No record found matching "'+find_name+'"')
            status = btrieveFile.GetLastStatusCode()
            if (status != 4):
                print(' Read error: ', status, ': ', btrv.Btrieve.StatusCodeToString(status))

# Close the file:
rc = btrieveClient.FileClose(btrieveFile)
if (rc == btrv.Btrieve.STATUS_CODE_NO_ERROR):
    print('File closed successfully!')
else:
    print('File close failed - status: ', rc, ': ', btrv.Btrieve.StatusCodeToString(status))

